@@ -174,7 +174,7 @@ def freeze(dist):
                      0, # no optimization
                      False,  # normal buffered output
                      len(code),
-                     ) + "\000" + code + "\000"
+                     ) + "\x00" + code + "\x00\x00"
     #  We try to bundle the python DLL into all bootstrap executables, even
     #  if it's not bundled in the frozen distribution.  This helps keep the
     #  bootstrap env small and minimises the chances of something going wrong.
@@ -337,6 +337,7 @@ def _chainload(target_dir):
           # Remove our own cruft from it before doing so.
           d_locals = d_globals = sys.modules["__main__"].__dict__
           d_locals.clear()
+          d_locals["__name__"] = "__main__"
           for code in codelist:
               exec code in d_globals, d_locals
           raise SystemExit(0)
