@@ -13,7 +13,7 @@ Example:
     app.install_version("1.2.3")
     -->   IOError:  permission denied
 
-    sapp = SudoWrapper(app)
+    sapp = SudoProxy(app)
     sapp.start()
     -->   prompts for credentials
     sapp.install_version("1.2.3")
@@ -29,32 +29,58 @@ We also provide some handy utility functions:
 
 """
 
+from __future__ import absolute_import
+
 import sys
-from functools import wraps
 
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
+from esky.util import LazyImport
 
+class LazyImport(LazyImport):
+    _esky_lazy_namespace = globals()
+
+class functools(LazyImport):
+    def _esky_lazy_import():
+        import functools
+        return functools
+
+class pickle(LazyImport):
+    def _esky_lazy_import():
+        try:
+           import cPickle as pickle
+        except ImportError:
+            import pickle
+        return pickle
 
-_impl = None
 
 if sys.platform == "win32":
-    from esky.sudo import sudo_win32 as _impl
+    class _impl(LazyImport):
+        def _esky_lazy_import():
+            from esky.sudo import sudo_win32 as _impl
+            return _impl
+elif sys.platform == "darwin":
+    class _impl(LazyImport):
+        def _esky_lazy_import():
+            try:
+                from esky.sudo import sudo_osx  as _impl
+            except ImportError:
+                from esky.sudo import sudo_unix as _impl
+            return _impl
 else:
-    if sys.platform == "darwin":
-        try:
-            from esky.sudo import sudo_osx  as _impl
-        except ImportError:
-            pass
-    if _impl is None:
-        from esky.sudo import sudo_unix as _impl
+    class _impl(LazyImport):
+        def _esky_lazy_import():
+            from esky.sudo import sudo_unix as _impl
+            return _impl
 
-spawn_sudo = _impl.spawn_sudo
-has_root = _impl.has_root
-can_get_root = _impl.can_get_root
-run_startup_hooks = _impl.run_startup_hooks
+
+def spawn_sudo(proxy):
+    return _impl.spawn_sudo(proxy)
+def has_root():
+    return _impl.has_root()
+def can_get_root():
+    return _impl.can_get_root()
+def run_startup_hooks():
+    if len(sys.argv) > 1 and sys.argv[1] == "--esky-spawn-sudo":
+        return _impl.run_startup_hooks()
 
 
 def b(data):
@@ -154,7 +180,7 @@ class SudoProxy(object):
         method = getattr(target,attr)
         pipe = self.__dict__["pipe"]
         if not _get_sudo_iterator(target,attr):
-            @wraps(method.im_func)
+            @functools.wraps(method.im_func)
             def wrapper(*args):
                 pipe.write(method.im_func.func_name.encode("ascii"))
                 for arg in args:
@@ -164,7 +190,7 @@ class SudoProxy(object):
                     raise result
                 return result
         else:
-            @wraps(method.im_func)
+            @functools.wraps(method.im_func)
             def wrapper(*args):
                 pipe.write(method.im_func.func_name.encode("ascii"))
                 for arg in args:
